#!/usr/bin/env python3
# --------------------------------------------------------------------------------------
# SPDX-FileCopyrightText: 2021 Magenta ApS <https://magenta.dk>
# SPDX-License-Identifier: MPL-2.0
# --------------------------------------------------------------------------------------
from typing import Any
from typing import Dict
from typing import Iterable
from typing import List
from typing import Optional
from typing import Tuple

from aiohttp import ClientResponseError
from fastapi.encoders import jsonable_encoder
from pydantic import AnyHttpUrl
from ramodels.mo import Employee
from ramodels.mo import FacetClass
from ramodels.mo import OrganisationUnit
from ramodels.mo._shared import MOBase
from ramodels.mo.details import Address
from ramodels.mo.details import Engagement
from ramodels.mo.details import EngagementAssociation
from ramodels.mo.details import ITUser
from ramodels.mo.details import Manager

from raclients.modelclientbase import ModelClientBase


class ModelClient(ModelClientBase):
    __mo_path_map: Dict[MOBase, str] = {
        OrganisationUnit: "/service/ou/create",
        Employee: "/service/e/create",
        Engagement: "/service/details/create",
        ITUser: "/service/details/create",
        EngagementAssociation: "/service/details/create",
        Manager: "/service/details/create",
        Address: "/service/details/create",
        FacetClass: "/service/f/{facet_uuid}/",
    }

    def __init__(
        self,
        base_url: AnyHttpUrl = AnyHttpUrl(
            "http://localhost:5000", scheme="http", host="localhost"
        ),
        force: bool = False,
        *args: Any,
        **kwargs: Optional[Any],
    ):
        super().__init__(base_url, *args, **kwargs)  # type: ignore
        self.force = int(force)

    def _get_healthcheck_tuples(self) -> List[Tuple[str, str]]:
        return [("/version/", "mo_version")]

    def _get_path_map(self) -> Dict[MOBase, str]:
        return self.__mo_path_map

    async def _post_single_to_backend(self, current_type: MOBase, obj: MOBase) -> Any:
        session = await self._verify_session()
        # Note that we additionally format the object's fields onto the path mapping to
        # support schemes such as /service/f/{facet_uuid}/, where facet_uuid is
        # retrieved from obj.facet_uuid.
        path = self.__mo_path_map[current_type].format_map(obj.dict())
        post_url = f"{self._base_url}{path}?force={self.force}"

        async with session.post(post_url, json=jsonable_encoder(obj)) as response:
            resp = await response.json()
            try:
                response.raise_for_status()
            except ClientResponseError as client_err:
                if "description" in resp:
                    client_err.message = resp["description"]
                raise client_err
            return resp

    async def load_mo_objs(
        self, objs: Iterable[MOBase], disable_progressbar: bool = False
    ) -> List[Any]:
        """
        lazy init client session to ensure created within async context
        :param objs:
        :param disable_progressbar:
        :return:
        """
        return await self._submit_payloads(
            objs, disable_progressbar=disable_progressbar
        )



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['raclients', 'raclients.graph', 'raclients.lora', 'raclients.mo']

package_data = \
{'': ['*']}

install_requires = \
['Authlib>=0.15.5,<0.16.0',
 'aiohttp>=3.7.4,<4.0.0',
 'fastapi>=0.70.0,<0.71.0',
 'gql==3.0.0b0',
 'httpx>=0.21.1,<0.22.0',
 'jsonschema>=3.2.0,<4.0.0',
 'more-itertools>=8.7.0,<9.0.0',
 'pydantic>=1.8.2,<2.0.0',
 'ra-utils[headers]>=0.4.0,<0.5.0',
 'ramodels>=3.26.1,<4.0.0',
 'structlog>=21.2.0,<22.0.0',
 'tqdm>=4.60.0,<5.0.0']

setup_kwargs = {
    'name': 'raclients',
    'version': '0.11.1',
    'description': 'Clients for OS2mo/LoRa',
    'long_description': '<!--\nSPDX-FileCopyrightText: 2021 Magenta ApS <https://magenta.dk>\nSPDX-License-Identifier: MPL-2.0\n-->\n\n\n# RA Clients\n\nOS2mo/LoRa clients\n\n<!--\n## Versioning\nThis project uses [Semantic Versioning](https://semver.org/) with the following strategy:\n- MAJOR: Incompatible changes to existing data models\n- MINOR: Backwards compatible updates to existing data models OR new models added\n- PATCH: Backwards compatible bug fixes\n\n## Getting Started\n\nTODO: README section missing!\n\n### Prerequisites\n\n\nTODO: README section missing!\n\n### Installing\n\nTODO: README section missing!\n\n## Running the tests\n\nTODO: README section missing!\n\n## Deployment\n\nTODO: README section missing!\n\n## Built With\n\nTODO: README section missing!\n\n## Authors\n\nMagenta ApS <https://magenta.dk>\n\nTODO: README section missing!\n-->\n## License\n- This project: [MPL-2.0](MPL-2.0.txt)\n- Dependencies:\n  - pydantic: [MIT](MIT.txt)\n\nThis project uses [REUSE](https://reuse.software) for licensing. All licenses can be found in the [LICENSES folder](LICENSES/) of the project.\n',
    'author': 'Magenta',
    'author_email': 'info@magenta.dk',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://magenta.dk/',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



#!/usr/bin/env python3
# --------------------------------------------------------------------------------------
# SPDX-FileCopyrightText: 2021 Magenta ApS <https://magenta.dk>
# SPDX-License-Identifier: MPL-2.0
# --------------------------------------------------------------------------------------
from typing import Any
from typing import cast
from typing import Dict
from typing import Iterable
from typing import List
from typing import Optional
from typing import Tuple
from typing import Type

from aiohttp import ClientResponseError
from aiohttp import ClientSession
from fastapi.encoders import jsonable_encoder
from jsonschema import validate
from pydantic import AnyHttpUrl
from ramodels.lora import Facet
from ramodels.lora import ITSystem
from ramodels.lora import Klasse
from ramodels.lora import Organisation
from ramodels.lora._shared import LoraBase

from raclients.modelclientbase import ModelClientBase


class ModelClient(ModelClientBase):
    __mox_path_map = {
        Organisation: "/organisation/organisation",
        Facet: "/klassifikation/facet",
        Klasse: "/klassifikation/klasse",
        ITSystem: "/organisation/itsystem",
    }

    def __init__(
        self,
        base_url: AnyHttpUrl = AnyHttpUrl(
            "http://localhost:8080", scheme="http", host="localhost"
        ),
        validate: bool = True,
        *args: Any,
        **kwargs: Optional[Any],
    ):
        self.validate: bool = validate
        self.schema_cache: Dict[LoraBase, Dict[str, Any]] = {}
        super().__init__(base_url, *args, **kwargs)  # type: ignore

    async def _fetch_schema(self, session: ClientSession, url: str) -> Dict[str, Any]:
        """Fetch jsonschema from LoRa."""
        response = await session.get(url)
        response.raise_for_status()
        return cast(Dict[str, Any], await response.json())

    async def _get_schema(
        self, session: ClientSession, current_type: Type[LoraBase]
    ) -> Dict[str, Any]:
        schema = self.schema_cache.get(current_type)
        if schema:
            return schema

        generic_url = self._base_url + self.__mox_path_map[current_type]
        url = generic_url + "/schema"
        schema = await self._fetch_schema(session, url)
        self.schema_cache[current_type] = schema
        return schema

    def _get_healthcheck_tuples(self) -> List[Tuple[str, str]]:
        return [("/version/", "lora_version")]

    def _get_path_map(self) -> Dict[LoraBase, str]:
        return self.__mox_path_map

    async def _post_single_to_backend(
        self, current_type: Type[LoraBase], obj: LoraBase
    ) -> Any:
        """

        :param current_type: Redundant, only pass it because we already have it
        :param obj:
        :return:
        """
        session: ClientSession = await self._verify_session()

        uuid = obj.uuid
        # TODO, PENDING: https://github.com/samuelcolvin/pydantic/pull/2231
        # for now, uuid is included, and has to be excluded when converted to json
        jsonified = jsonable_encoder(
            obj=obj, by_alias=True, exclude={"uuid"}, exclude_none=True
        )
        generic_url = self._base_url + self.__mox_path_map[current_type]

        if self.validate:
            schema = await self._get_schema(session, current_type)
            validate(instance=jsonified, schema=schema)
        assert uuid is not None
        async with session.put(generic_url + f"/{uuid}", json=jsonified) as response:
            resp_json = await response.json()
            try:
                response.raise_for_status()
            except ClientResponseError as client_err:
                if "description" in resp_json:
                    client_err.message = resp_json["description"]
                raise client_err
            return resp_json

    async def load_lora_objs(
        self, objs: Iterable[LoraBase], disable_progressbar: bool = False
    ) -> List[Any]:
        """Lazy init client session to ensure created within async context

        Args:
            objs (Iterable[LoraBase]): LoRa objects to load
            disable_progressbar (bool, optional): Whether to disable the progress bar.
                Defaults to False.

        Returns:
            List[Any]: List of JSON responses.
        """
        return await self._submit_payloads(
            objs, disable_progressbar=disable_progressbar
        )
